@@ -14,16 +14,15 @@ BEGIN {
 
 use File::HomeDir;
 use Mojo::URL;
-use Test::Mojo;
 use Mojo::Loader;
 use Mojo::UserAgent;
 use base qw( Test::Builder::Module );
-use Carp qw( croak );
+use Carp qw( croak carp );
 use File::Basename ();
 use File::Path ();
 
 # ABSTRACT: Test an imaginary beowulf cluster of Clustericious services
-our $VERSION = '0.25'; # VERSION
+our $VERSION = '0.26'; # VERSION
 
 
 BEGIN { $ENV{MOJO_LOG_LEVEL} = 'fatal' }
@@ -38,6 +37,7 @@ sub new
   {
     # undocumented and deprecated
     # you can pass in just an instance of Test::Mojo
+    carp "passing a Test::Mojo object into Test::Clustericious::Cluster->new deprecated";
     $args = { t => $_[0] };
   }
   else
@@ -45,8 +45,6 @@ sub new
     $args = ref $_[0] ? { %{ $_[0] } } : {@_};
   }
 
-  my $t = $args->{t} // Test::Mojo->new;
-  
   my $sep = $^O eq 'MSWin32' ? ';' : ':';
   my $lite_path = [ split $sep, $ENV{PATH} ];
 
@@ -54,7 +52,7 @@ sub new
   unshift @$lite_path, ref($args->{lite_path}) ? @{ $args->{lite_path} } : ($args->{lite_path});
   
   bless { 
-    t           => $t, 
+    #t           => $t, 
     urls        => [], 
     apps        => [], 
     stopped     => [],
@@ -63,13 +61,33 @@ sub new
     servers     => [],
     app_servers => [],
     auth_url    => '',
-    extra_ua    => [$t->ua],
+    #extra_ua    => [$t->ua],
     lite_path   => $lite_path,
   }, $class;
 }
 
 
-sub t { shift->{t} }
+sub t {
+  my($self) = @_;
+  $self->{t} //= do {
+    require Test::Mojo;
+    my $t = $self->{t} = Test::Mojo->new;
+    $self->_update_default_server;
+    $t;
+  };
+}
+
+sub _update_default_server
+{
+  my($self) = @_;
+  $self->t->ua->server->app($self->apps->[-1])
+    if $self->{t} && @{ $self->apps } && !$self->{stopped}->[-1];
+}
+
+sub _extra_ua {
+  my($self) = @_;
+  $self->{extra_ua} //= [ $self->t->ua ];
+}
 
 
 sub urls { shift->{urls} }
@@ -87,31 +105,37 @@ sub url { shift->{url} }
 sub auth_url { shift->{auth_url} }
 
 
-our $inc_hook;
-
 BEGIN {
+  # TODO
+  # so we muck with this @INC in two places.  Here we do it so
+  # that you can use_ok files in your .t file.  Later we do 
+  # extract files in create_cluster_ok and add ~/lib to the @INC
+  # path so that we can load as regular files.  This is more
+  # reliable for anything that expects a real live file, and we'd
+  # like to do that for use_ok as well in the future.
+
   unshift @INC, sub {
     my($self, $file) = @_;
 
-    return $inc_hook->($self, $file) if $inc_hook;
-   
+    # avoid deep recursion
     state $first;
-    unless($first)
-    {
-      $first = 1;
-      Mojo::Loader::load_class('main');
-    }
+    Mojo::Loader::load_class('main') unless $first++;
   
     my $data = Mojo::Loader::data_section('main', "lib/$file");
     return unless defined $data;
+    
+    # This will make the file really there.
+    # Some stuff depends on that
+    __PACKAGE__->extract_data_section("lib/$file", 'main');
+
     open my $fh, '<', \$data;
   
     # fake out %INC because Mojo::Home freeks the heck
     # out when it sees a CODEREF on some platforms
     # in %INC
     my $home = File::HomeDir->my_home;
-    mkdir "$home/mojohome" unless -d "$home/mojohome";
-    $INC{$file} = "$home/mojohome/$file";
+    mkdir "$home/lib" unless -d "$home/lib";
+    $INC{$file} = "$home/lib/$file";
   
     return $fh;
   };
@@ -143,7 +167,7 @@ sub _add_app_to_ua
 sub _add_app
 {
   my($self, $url, $app, $index) = @_;
-  $self->_add_app_to_ua($_, $url, $app, $index) for @{ $self->{extra_ua} };
+  $self->_add_app_to_ua($_, $url, $app, $index) for @{ $self->_extra_ua };
   return;
 }
 
@@ -165,7 +189,7 @@ sub _add_ua
     next if $stopped;
     $self->_add_app_to_ua($ua, $self->{urls}->[$i], $self->{apps}->[$i], $i);
   }
-  push @{ $self->{extra_ua} }, $ua;
+  push @{ $self->_extra_ua }, $ua;
   return $ua;
 }
 
@@ -217,22 +241,12 @@ sub create_cluster_ok
   my $caller = caller;
   Mojo::Loader::load_class($caller);
 
-  local $inc_hook = sub {
-    my($self, $file) = @_;
-    my $data = Mojo::Loader::data_section($caller, "lib/$file");
-    return unless defined $data;
-    open my $fh, '<', \$data;
-    
-    # fake out %INC because Mojo::Home freeks the heck
-    # out when it sees a CODEREF on some platforms
-    # in %INC
-    my $home = File::HomeDir->my_home;
-    mkdir "$home/mojohome" unless -d "$home/mojohome";
-    $INC{$file} = "$home/mojohome/$file";
-    
-    return $fh;
-  };
-  
+  local @INC = @INC;
+  $self->extract_data_section(qr{^lib/}, $caller);
+  my $home = File::HomeDir->my_home;
+  unshift @INC, "$home/lib"
+    if -d "$home/lib";
+
   foreach my $i (0..$#_)
   {
     $self->{index}++;
@@ -330,6 +344,17 @@ sub create_cluster_ok
         if(my $error = $@)
         { push @errors, [ $app_name, $error ] }
       }
+      
+      if(my $script = Mojo::Loader::data_section($caller, "script/$app_name.psgi"))
+      {
+        my $home = File::HomeDir->my_home;
+        require Mojolicious;
+        require Mojolicious::Plugin::MountPSGI;
+        $app = Mojolicious->new;
+        # TODO: check syntax of .psgi file?
+        $self->extract_data_section("script/$app_name.psgi", $caller);
+        $app->plugin('Mojolicious::Plugin::MountPSGI' => { '/' => "$home/script/$app_name.psgi" });
+      }
     }
     
     unless(defined $app)
@@ -384,6 +409,11 @@ sub create_cluster_ok
     }
   }
   
+  if(defined $self->{t})
+  {
+    $self->_update_default_server;
+  }
+  
   return $self;
 }
 
@@ -447,6 +477,12 @@ sub stop_ok
     eval { @{ $self->{app_servers}->[$index] } = () };
     $error = $@;
     $ok = 0 if $error;
+    
+    if($index == $#{ $self->apps } && $self->{t})
+    {
+      $self->t->ua->server(Mojo::UserAgent::Server->new);
+    }
+    
   }
   else
   {
@@ -485,6 +521,7 @@ sub start_ok
       $tb->diag("error in start: $error");
       $ok = 0;
     }
+
   }
   else
   {
@@ -494,6 +531,11 @@ sub start_ok
   
   $self->{stopped}->[$index] = 0 if $ok;
 
+  if($index == $#{ $self->apps } && $self->{t})
+  {
+    $self->_update_default_server;
+  }
+
   $test_name //= "start service ($index)";
   
   $tb->ok($ok, $test_name);
@@ -534,10 +576,17 @@ sub create_ua
 
 sub extract_data_section
 {
-  my($self, $regex) = @_;
+  my($class, $regex, $caller) = @_;
 
   $regex //= qr{};
-  my $caller = caller;
+  
+  unless(ref $regex eq 'Regexp')
+  {
+    $regex = quotemeta $regex;
+    $regex = qr{^$regex$};
+  }
+  
+  $caller //= caller;
   my $all = Mojo::Loader::data_section $caller;
   my $home = File::HomeDir->my_home;
   my $tb = __PACKAGE__->builder;
@@ -549,15 +598,21 @@ sub extract_data_section
     my $basename = File::Basename::basename $name;
     my $dir      = File::Basename::dirname  $name;
 
-    $tb->note("[extract] DIR  $home/dir");
-    File::Path::mkpath "$home/$dir", 0, 0700;
-    $tb->note("[extract] FILE $home/$dir/$basename");
-    open my $fh, '>', "$home/$dir/$basename";
-    print $fh $all->{$name};
-    close $fh;
+    unless(-d "$home/$dir")
+    {
+      $tb->note("[extract] DIR  $home/$dir");
+      File::Path::mkpath "$home/$dir", 0, 0700;
+    }
+    unless(-f "$home/$dir/$basename")
+    {
+      $tb->note("[extract] FILE $home/$dir/$basename");
+      open my $fh, '>', "$home/$dir/$basename";
+      print $fh $all->{$name};
+      close $fh;
+    }
   }
   
-  $self;
+  $class;
 }
 
 1;
@@ -574,7 +629,7 @@ Test::Clustericious::Cluster - Test an imaginary beowulf cluster of Clustericiou
 
 =head1 VERSION
 
-version 0.25
+version 0.26
 
 =head1 SYNOPSIS
 
@@ -796,6 +851,12 @@ lite application.  The script for the lite app must be executable.
 You can specify additional directories to search using the
 C<lite_path> argument to the constructor.
 
+This can also be a PSGI application.  In this case it needs to be
+in the C<__DATA__> section of your test and it must have a name
+in the form C<script/app.psgi>.  This also requires
+L<Mojolicious::Plugin::MountPSGI> already be installed so if you
+use this feature make sure you declare that as a prereq.
+
 =item list reference in the form: [ string, hashref ]
 
 The string is taken to be the L<Mojolicious> application name.
@@ -888,9 +949,11 @@ to connect to nodes in the test cluster.
 =head2 extract_data_section
 
  $cluster->extract_data_section($regex);
+ Test::Clustericious::Cluster->extract_data_section($regex);
 
 Extract the files from the data section of the current package
-that match the given regex.
+that match the given regex.  C<$regex> can also be a plain
+string for an exact filename match.
 
 =head1 CAVEATS
 
